//+------------------------------------------------------------------+
//|                                                FractalLevels.mq4 |
//|                                        Copyright  2008, lotos4u |
//|                                                lotos4u@gmail.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, lotos4u"
#property link      "lotos4u@gmail.com"

#property indicator_chart_window
#property indicator_buffers 4

#property indicator_width1 4
#property indicator_width2 4
#property indicator_width3 2
#property indicator_width4 2

#property indicator_color1 Red
#property indicator_color2 Blue
#property indicator_color3 Gold
#property indicator_color4 Gold

//---- input parameters
extern int  FractalBars     = 3;
extern bool alertsOn        = true;
extern bool alertsOnCurrent = true;
extern bool alertsMessage   = true;
extern bool alertsSound     = false;
extern bool alertsEmail     = false;

//---- buffers
double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ArrowUpBuffer3[];
double ArrowDownBuffer4[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   SetIndexBuffer(0, ExtMapBuffer1);
   SetIndexLabel(0, " ");
   
   SetIndexBuffer(1, ExtMapBuffer2);
   SetIndexLabel(1, " ");

   SetIndexStyle(2, DRAW_ARROW);
   SetIndexArrow(2, 119);
   SetIndexBuffer(2, ArrowUpBuffer3);
   SetIndexLabel(2, " ");
   
   SetIndexStyle(3, DRAW_ARROW);
   SetIndexArrow(3, 119);
   SetIndexBuffer(3, ArrowDownBuffer4);
   SetIndexLabel(3, " ");
   return(0);
}


//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit(){return(0);}



//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars = IndicatorCounted();
   int limit;


   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
       limit = Bars - counted_bars;

   for(int i = limit; i > 0; i--)
   {
      ArrowUpBuffer3[i]  = EMPTY_VALUE;
      ArrowDownBuffer4[i]= EMPTY_VALUE;
      ExtMapBuffer1[i] = isFractalUp(i, FractalBars);
      if(ExtMapBuffer1[i] == 0)
         ExtMapBuffer1[i] = ExtMapBuffer1[i+1];
      else
         ArrowUpBuffer3[i] = ExtMapBuffer1[i];
         
      
      ExtMapBuffer2[i] = isFractalDown(i, FractalBars);
      if(ExtMapBuffer2[i] == 0)
         ExtMapBuffer2[i] = ExtMapBuffer2[i+1];
      else
         ArrowDownBuffer4[i] = ExtMapBuffer2[i];
         
   }
   manageAlerts();
   return(0);
}


double isFractalUp(int index, int bars)
{
   double max = High[index];
   for(int i = index + bars; i >= (index - bars); i--)
   {
      if(High[i] == 0.0)return(0);
      if(max <= High[i] && i != index)
      {
         if(max < High[i])
            return(0);
         if(MathAbs(i - index) > 1)
            return(0);
      }
   }
   return(max);
}


double isFractalDown(int index, int bars)
{
   double min = Low[index];
   for(int i = index + bars; i >= (index - bars); i--)
   {
      if(Low[i] == 0.0)return(0);
      if(min >= Low[i] && i != index)
      {
         if(min > Low[i])
            return(0);

         if(MathAbs(i - index) > 1)
            return(0);
      }
   }
   return(min);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = FractalBars;
      else     whichBar = FractalBars+1;
      if (ArrowUpBuffer3[whichBar] != EMPTY_VALUE || ArrowDownBuffer4[whichBar] != EMPTY_VALUE)
      {
         if (ArrowUpBuffer3[whichBar]   != EMPTY_VALUE) doAlert(whichBar,"sell");
         if (ArrowDownBuffer4[whichBar] != EMPTY_VALUE) doAlert(whichBar,"buy");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," FractalLevels fractal changed to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," FractalLevels"),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

